#include "pdp2.h"
#include "xgrafix.h"
#include <string.h>

void XGWrite(void *, int, int, FILE *, char *); 
void XGRead(void *, int, int, FILE *, char *);


/****************************************************************/

void InitWindows()
{
  register int i, j, k;
  char buffer[50];
  
  /*********************************************/
  /* Set up each window structure              */
  
  XGSet3D("linlinlin", "X", "Y", "Phi(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, phi, &ngx, &ngy, 3);
  
  /*********************************************/
  
  XGSet3D("linlinlin", "X", "Y", "Phi_pois(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, phi_pois, &ngx, &ngy, 3);
  
  /*********************************************/
  
  XGSet3D("linlinlin", "X", "Y", "Sigma(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, sigma, &ngx, &ngy, 3);
  
  /*********************************************/
  
  XGSet3D("linlinlin", "X", "Y", "Rho(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, rho, &ngx, &ngy, 3);
  
  /*********************************************/
  
  if(ybc_flag == PERIODIC) {
    XGSet3D("linlinlin", "X", "Ky", "|Phi(x,Ky)|", 45.0, 45.0, "closed", 100, 150,
	    1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, khi, 0.0, 0.0);
    
    XGSurf(x_array, k_array, show_phik, &ngx, &khi_plus1, 3);
    
    /*******************************************/
    
    XGSet3D("linlinlin", "Ky", "freq.", "|Phi(Ky,f)|", 45.0, 45.0, "closed", 100, 150,
	    1.0, 1.0, 1.0, False, False, True, 0.0, khi, 0.0, freq_hi*df, 0.0, 0.0);
    
    XGSurf(k_array, f_array, phi_mid_k_f, &khi_plus1, &freq_hi, 3);
    
    /*******************************************/
    
    XGSet3D("linlinlin", "X", "Ky", "|Rho(x,Ky)|", 45.0, 45.0, "closed", 100, 150,
	    1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, khi, 0.0, 0.0);
    
    XGSurf(x_array, k_array, show_rhok, &ngx, &khi_plus1, 3);
  }
  /*********************************************/
  
  XGSet3D("linlinlin", "X", "Y", "Ex(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, ex, &ngx, &ngy, 3);
  
  /*********************************************/
  
  XGSet3D("linlinlin", "X", "Y", "Ey(x,y)", 45.0, -120.0, "closed", 100, 150,
	  1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
  XGSurf(x_array, y_array, ey, &ngx, &ngy, 3);
  
  /*********************************************/
  
  for(j=0; j<nstrctype; j++){
    sprintf(buffer, "Particles collected on %s", strc_name[j]);
    XGSet2D("linlin", "Time", buffer, "closed", 100, 400, 1.0, 1.0, 
	    True, True, 0.0, 0.0, 0.0, 0.0);
    
    for (i=0; i<nsp; i++) XGCurve(t_array, stc_np_hist[j][i], &hist_hi, i);
  }
  
  /*********************************************/
  
  XGSetVec("vecvec", "X Axis", "Y Axis", "E(x,y)", "closed",
	   700, 175, 1.0, 1.0, False, False,
	   (lhs_flag)? -dx:0, (rhs_flag)? xlength+dx:xlength, 0.0, ylength);
  
  /***  Internal Structures ****/
  for(k=0; k<nstrcmax; k++)
    XGStructure(5, FILLED, 4, 3+strc[k].type, strc[k].xl*dx, strc[k].yl*dy,
		strc[k].xr*dx, strc[k].yl*dy, strc[k].xr*dx, strc[k].yu*dy,
		strc[k].xl*dx, strc[k].yu*dy, strc[k].xl*dx, strc[k].yl*dy);
  
  /***  Left Electrode ****/
  if(lhs_flag) {
    XGStructure(5, FILLED, 4, 7, -dx, lhs_nys*dy, 0.0, lhs_nys*dy, 0.0, lhs_nyf*dy,
		-dx, lhs_nyf*dy, -dx, lhs_nys*dy);
    XGStructure(5, FILLED, 4, 8, -dx, 0.0, 0.0, 0.0, 0.0, (lhs_nys-lhs_gap)*dy,
		-dx, (lhs_nys-lhs_gap)*dy, -dx, 0.0);
    XGStructure(5, FILLED, 4, 8, -dx, ngy*dy, 0.0, ngy*dy, 0.0, (lhs_nyf+lhs_gap)*dy,
		-dx, (lhs_nyf+lhs_gap)*dy, -dx, ngy*dy);
  }
  
  /***  Right Electrode ****/
  if(rhs_flag) {
    XGStructure(5, FILLED, 4, 7, xlength+dx, rhs_nys*dy, xlength, rhs_nys*dy,
		xlength, rhs_nyf*dy, xlength+dx, rhs_nyf*dy, xlength+dx, rhs_nys*dy);
    XGStructure(5, FILLED, 4, 8, xlength+dx, 0.0, xlength, 0.0, xlength, (rhs_nys-rhs_gap)*dy,
		xlength+dx, (rhs_nys-rhs_gap)*dy, xlength+dx, 0.0);
    XGStructure(5, FILLED, 4, 8, xlength+dx, ngy*dy, xlength, ngy*dy,
		xlength, (rhs_nyf+rhs_gap)*dy, xlength+dx, (rhs_nyf+rhs_gap)*dy,
		xlength+dx, ngy*dy);
  }
  
  XGVector(x_array, y_array, ex, ey, &ngx, &ngy, 3);
  
  /*********************************************/
  
  if(nsp) {
    XGSet2D("linlin", "Time", "Number(t)", "closed", 100, 400, 1.0, 1.0, 
	    True, True, 0.0, 0.0, 0.0, 0.0);
    
    for (i=0; i<nsp; i++) XGCurve(t_array, np_hist[i], &hist_hi, i);
    
    /*******************************************/
    if(wall_flag) {
      for(k=0; k<SIDES; k++) {
	
	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Profile(y) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "Y", buffer, "closed", 200, 100,
		  1.0, 1.0, False, True, 0.0, ylength, 0.0, 0.0);
	  
	  XGCurve(y_array, wall_q[k][i], &ngy, 8);
	}
	
	/*******************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Dist(y,E) %s SIDE %d", name[i], k+1);
	  XGSet3D("linlinlin", "Y", "E", buffer, 45.0, -45.0,
		  "closed", 100, 150, 1.0, 1.0, 1.0, False, False, True,
		  0.0, ylength, e_array[k][i][0], e_array[k][i][nbin[k][i]-1], 0.0, 0.0);
	  
	  XGSurf(y_array, e_array[k][i], fe[k][i], &ngy, &nbin[k][i], 3);
	}
	
	/*******************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Dist(y,theta) %s SIDE %d", name[i], k+1);
	  XGSet3D("linlinlin", "Y", "Theta", buffer, 45.0, -45.0,"closed",
		  100, 150, 1.0, 180.0/M_PI, 1.0, False, False, True, 0.0, ylength,
		  th_array[k][i][0]*180.0/M_PI, th_array[k][i][nbin[k][i]-1]*180.0/M_PI, 0.0, 0.0);
	  
	  XGSurf(y_array, th_array[k][i], fth[k][i], &ngy, &nbin[k][i], 3);
	}
	
	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "E_ave(y) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "Y", buffer, "closed", 600, 100,
		  1.0, 1.0, False, True, 0.0, ylength, 0.0, 0.0);
	  
	  XGCurve(y_array, e_ave_show[k][i], &ngy, 8);
	}
	
	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Theta_ave(y) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "Y", buffer, "closed", 600, 100,
		  1.0, 180.0/M_PI, False, True, 0.0, ylength, 0.0, 0.0);
	  
	  XGCurve(y_array, th_ave_show[k][i], &ngy, 8);
	}
	
	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Center Dist(E) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "E", buffer, "closed", 600, 100,
		  1.0, 1.0, False, True, e_array[k][i][0],
		  e_array[k][i][nbin[k][i]-1], 0.0, 0.0);
	  
	  XGCurve(e_array[k][i], fetot_center[k][i], &nbin[k][i], 8);
	}

	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Edge Dist(E) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "E", buffer, "closed", 600, 100,
		  1.0, 1.0, False, True, e_array[k][i][0],
		  e_array[k][i][nbin[k][i]-1], 0.0, 0.0);
	  
	  XGCurve(e_array[k][i], fetot_edge[k][i], &nbin[k][i], 8);
	}

	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Center Dist(theta) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "Theta", buffer, "closed", 600, 100,
		  180.0/M_PI, 1.0, False, True, th_array[k][i][0]*180.0/M_PI,
		  th_array[k][i][nbin[k][i]-1]*180.0/M_PI, 0.0, 0.0);
	  	  
	  XGCurve(th_array[k][i], fthtot_center[k][i], &nbin[k][i], 8);
	}
	/*********************************************/
	
	for(i=0; i<nsp; i++) {
	  sprintf(buffer, "Edge Dist(theta) %s SIDE %d", name[i], k+1);
	  XGSet2D("linlin", "Theta", buffer, "closed", 600, 100,
		  180.0/M_PI, 1.0, False, True, th_array[k][i][0]*180.0/M_PI,
		  th_array[k][i][nbin[k][i]-1]*180.0/M_PI, 0.0, 0.0);
	  
	  XGCurve(th_array[k][i], fthtot_edge[k][i], &nbin[k][i], 8);
	}
      }
    }
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Y-X Phase Space %s", name[i]);
      XGSet2D("linlin", "X", buffer, "closed", 300, 10, dx, dy, False, False,
	      (lhs_flag)? -dx:0, (rhs_flag)? xlength+dx:xlength, 0.0, ylength);
      
      /***  Internal Structures ****/
      for(k=0; k<nstrcmax; k++)
	XGStructure(5, FILLED, 4, 3+strc[k].type, strc[k].xl*dx, strc[k].yl*dy,
		    strc[k].xr*dx, strc[k].yl*dy, strc[k].xr*dx, strc[k].yu*dy,
		    strc[k].xl*dx, strc[k].yu*dy, strc[k].xl*dx, strc[k].yl*dy);
     
      /***  Left Electrode ****/
      if(lhs_flag) {
	XGStructure(5, FILLED, 4, 7, -dx, lhs_nys*dy, 0.0, lhs_nys*dy, 0.0, lhs_nyf*dy,
		    -dx, lhs_nyf*dy, -dx, lhs_nys*dy);
	XGStructure(5, FILLED, 4, 8, -dx, 0.0, 0.0, 0.0, 0.0, (lhs_nys-lhs_gap)*dy,
		    -dx, (lhs_nys-lhs_gap)*dy, -dx, 0.0);
	XGStructure(5, FILLED, 4, 8, -dx, ngy*dy, 0.0, ngy*dy, 0.0, (lhs_nyf+lhs_gap)*dy,
		    -dx, (lhs_nyf+lhs_gap)*dy, -dx, ngy*dy);
      }
      
      /***  Right Electrode ****/
      if(rhs_flag) {
	XGStructure(5, FILLED, 4, 7, xlength+dx, rhs_nys*dy, xlength, rhs_nys*dy,
		    xlength, rhs_nyf*dy, xlength+dx, rhs_nyf*dy, xlength+dx, rhs_nys*dy);
	XGStructure(5, FILLED, 4, 8, xlength+dx, 0.0, xlength, 0.0, xlength, (rhs_nys-rhs_gap)*dy,
		    xlength+dx, (rhs_nys-rhs_gap)*dy, xlength+dx, 0.0);
	XGStructure(5, FILLED, 4, 8, xlength+dx, ngy*dy, xlength, ngy*dy,
		    xlength, (rhs_nyf+rhs_gap)*dy, xlength+dx, (rhs_nyf+rhs_gap)*dy,
		    xlength+dx, ngy*dy);
      }
      
      XGScat2D(x[i], y[i], &np[i], i);
    }
    
    /*******************************************/
    
    XGSet2D("linlin", "X", "Vx-X Phase Space", "closed", 200, 200, dx, vxscale[0],
	    False, True, 0.0, xlength, 0.0, 0.0);
    for(i=0; i<nsp; i++)
      XGScat2D(x[i], vx[i], &np[i], i);
    
    /*******************************************/

    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vx-X Phase Space %s", name[i]);
      XGSet2D("linlin", "X", buffer, "closed", 200, 200, dx, vxscale[i],
	      False, True, 0.0, xlength, 0.0, 0.0);
      
      XGScat2D(x[i], vx[i], &np[i], i);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vy-X Phase Space %s", name[i]);
      XGSet2D("linlin", "X", buffer, "closed", 200, 200, dx, vyscale[i],
	      False, True, 0.0, xlength, 0.0, 0.0);
      
      XGScat2D(x[i], vy[i], &np[i], i);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vz-X Phase Space %s", name[i]);
      XGSet2D("linlin", "X", buffer, "closed", 200, 200, dx, vzscale[i],
	      False, True, 0.0, xlength, 0.0, 0.0);
      
      XGScat2D(x[i], vz[i], &np[i], i);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vx-Vy Phase Space %s", name[i]);
      XGSet2D("linlin", "Vy", buffer, "closed", 500, 100, vyscale[i], vxscale[i],
	      True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGScat2D(vy[i], vx[i], &np[i], i);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "N(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
	      100, 150, 1.0, 1.0, nc2p/zlength/dx/dy, False, False, True,
	      0.0, xlength, 0.0, ylength, 0.0, 0.0);
      
      XGSurf(x_array, y_array, sp_n[i], &ngx, &ngy, 3);
    }
    
    /*******************************************/

    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Sigma(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
              100, 150, 1.0, 1.0, nc2p/zlength/dx/dy, False, False, True,
              0.0, xlength, 0.0, ylength, 0.0, 0.0);

      XGSurf(x_array, y_array, sp_sigma[i], &ngx, &ngy, 3);
    }

    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "V0(x,y) %s", name[i]);
      XGSetVec("vecvec", "X Axis", "Y Axis", buffer, "closed",
	       700, 175, 1.0, 1.0, False, False,
	       (lhs_flag)? -dx:0, (rhs_flag)? xlength+dx:xlength, 0.0, ylength);
      
      /***  Internal Structures ****/
      for(k=0; k<nstrcmax; k++)
	XGStructure(5, FILLED, 4, 3+strc[k].type, strc[k].xl*dx, strc[k].yl*dy,
		    strc[k].xr*dx, strc[k].yl*dy, strc[k].xr*dx, strc[k].yu*dy,
		    strc[k].xl*dx, strc[k].yu*dy, strc[k].xl*dx, strc[k].yl*dy);
      
      /***  Left Electrode ****/
      if(lhs_flag) {
	XGStructure(5, FILLED, 4, 7, -dx, lhs_nys*dy, 0.0, lhs_nys*dy, 0.0, lhs_nyf*dy,
		    -dx, lhs_nyf*dy, -dx, lhs_nys*dy);
	XGStructure(5, FILLED, 4, 8, -dx, 0.0, 0.0, 0.0, 0.0, (lhs_nys-lhs_gap)*dy,
		    -dx, (lhs_nys-lhs_gap)*dy, -dx, 0.0);
	XGStructure(5, FILLED, 4, 8, -dx, ngy*dy, 0.0, ngy*dy, 0.0, (lhs_nyf+lhs_gap)*dy,
		    -dx, (lhs_nyf+lhs_gap)*dy, -dx, ngy*dy);
      }
      
      /***  Right Electrode ****/
      if(rhs_flag) {
	XGStructure(5, FILLED, 4, 7, xlength+dx, rhs_nys*dy, xlength, rhs_nys*dy,
		    xlength, rhs_nyf*dy, xlength+dx, rhs_nyf*dy, xlength+dx, rhs_nys*dy);
	XGStructure(5, FILLED, 4, 8, xlength+dx, 0.0, xlength, 0.0, xlength, (rhs_nys-rhs_gap)*dy,
		    xlength+dx, (rhs_nys-rhs_gap)*dy, xlength+dx, 0.0);
	XGStructure(5, FILLED, 4, 8, xlength+dx, ngy*dy, xlength, ngy*dy,
		    xlength, (rhs_nyf+rhs_gap)*dy, xlength+dx, (rhs_nyf+rhs_gap)*dy,
		    xlength+dx, ngy*dy);
      }
      XGVector(x_array, y_array, sp_vx0[i], sp_vy0[i], &ngx, &ngy, 3);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vx0(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
	      100, 150, 1.0, 1.0, 1.0, False, False, True,
	      0.0, xlength, 0.0, ylength, 0.0, 0.0);
      
      XGSurf(x_array, y_array, sp_vx0[i], &ngx, &ngy, 3);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vy0(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
	      100, 150, 1.0, 1.0, 1.0, False, False, True,
	      0.0, xlength, 0.0, ylength, 0.0, 0.0);
      
      XGSurf(x_array, y_array, sp_vy0[i], &ngx, &ngy, 3);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vz0(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
	      100, 150, 1.0, 1.0, 1.0, False, False, True,
	      0.0, xlength, 0.0, ylength, 0.0, 0.0);
      
      XGSurf(x_array, y_array, sp_vz0[i], &ngx, &ngy, 3);
    }
    
    /*******************************************/
    
    for(i=0; i<nsp; i++) {
      sprintf(buffer, "Vt(x,y) %s", name[i]);
      XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0, "closed",
	      100, 150, 1.0, 1.0, 1.0, False, False, True,
	      0.0, xlength, 0.0, ylength, 0.0, 0.0);
      
      XGSurf(x_array, y_array, sp_vt[i], &ngx, &ngy, 3);
    }
  }
  /*******************************************/
  
  XGSet2D("linlin", "Time", "MID V(t)", "closed", 100, 500,
	  1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
  
  XGCurve(t_array, com_phi_mid_hist, &hist_hi, 2);
  
  /*******************************************/
  
  if(lhs_flag) {
    
    XGSet3D("linlinlin", "X", "Y", "Phi_lap_lhs(x,y)", 45.0, -120.0, "closed", 100, 150,
	    1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
    
    XGSurf(x_array, y_array, phi_lap_lhs, &ngx, &ngy, 3);
    
    /*********************************************/
    
    XGSet2D("linlin", "Y", "LHS Sigma(y)", "closed", 300, 400,
	    1.0, 1.0, False, True, 0.0, ylength, 0.0, 0.0);
    
    XGCurve(y_array, sigma[0], &ngy, 8);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "LHS V(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phi_lhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "LHS V_c(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phc_lhs_hist, &hist_hi, 3);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "LHS V_s(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phs_lhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "LHS I(t)", "closed", 500, 50,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_cur_lhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "LHS P(t)", "closed", 500, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_pow_lhs_hist, &hist_hi, 2);
  }
  /*********************************************/

  if(rhs_flag) {

    XGSet3D("linlinlin", "X", "Y", "Phi_lap_rhs(x,y)", 45.0, -120.0, "closed", 100, 150,
	    1.0, 1.0, 1.0, False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
    
    XGSurf(x_array, y_array, phi_lap_rhs, &ngx, &ngy, 3);
    
    /*********************************************/
    
    XGSet2D("linlin", "Y", "RHS Sigma(y)", "closed", 300, 400,
	    1.0, 1.0, False, True, 0.0, ylength, 0.0, 0.0);
    
    XGCurve(y_array, sigma[ncx], &ngy, 8);

    /*******************************************/
    
    XGSet2D("linlin", "Time", "RHS V(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phi_rhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "RHS V_c(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phc_rhs_hist, &hist_hi, 3);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "RHS V_s(t)", "closed", 100, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_phs_rhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "RHS I(t)", "closed", 500, 50,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_cur_rhs_hist, &hist_hi, 2);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "RHS P(t)", "closed", 500, 500,
	    1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(t_array, com_pow_rhs_hist, &hist_hi, 2);
  }
  
  /*********************************************/

  if(nfft) {
    if(nsp) {
      for(i=0; i<nsp; i++) {
	sprintf(buffer, "N_ave(x,y) %s", name[i]);
	XGSet3D("linlinlin", "X", "Y", buffer, 45.0, -120.0,
		"closed", 100, 150, 1.0, 1.0, nc2p/zlength/dx/dy,
		False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
	
	XGSurf(x_array, y_array, sp_n_ave_show[i], &ngx, &ngy, 3);
      }
    }
    /*******************************************/
    
    XGSet3D("linlinlin", "X", "Y", "Phi_ave(x,y)", 45.0, -120.0,
	    "closed", 100, 150, 1.0, 1.0, 1.0, False, False, 
	    True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
    
    XGSurf(x_array, y_array, phi_ave_show, &ngx, &ngy, 3);
    
    /*******************************************/
    
    XGSet2D("linlin", "Time", "Local MID V(t)", "closed",
	    100, 500, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(Local_t_array, phi_mid_hist, &thist_hi, 2);
    
    /*****************************************/
    
    XGSet2D("linlog", "Freq", "Mag of MID V(f)", "closed",
	    100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
    
    XGCurve(f_array, phi_mid_fft, &freq_hi, 4);
    
    /*****************************************/
    
    XGSet2D("linlin", "Freq", "Phase of MID V(f)", "closed",
	    400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
    
    XGCurve(f_array, phi_mid_fft+freq_hi, &freq_hi, 4);
    
    /*****************************************/
    
    if(lhs_flag) {
      XGSet2D("linlin", "Time", "Local LHS V(t)", "closed",
	      100, 500, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, phi_lhs_hist, &thist_hi, 2);
    
      /*****************************************/
      
      XGSet2D("linlin", "Time", "Local LHS I(t)", "closed",
	      500, 50, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, cur_lhs_hist, &thist_hi, 2);
      
      /*****************************************/
      
      XGSet2D("linlin", "Time", "Local LHS P(t)", "closed",
	      500, 500, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, pow_lhs_hist, &thist_hi, 2);

      /*****************************************/

      XGSet2D("linlog", "Freq", "Mag of LHS V(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, phi_lhs_fft, &freq_hi, 4);
      
      /*****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of LHS V(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, phi_lhs_fft+freq_hi, &freq_hi, 4);

      /*****************************************/

      XGSet2D("linlog", "Freq", "Mag of LHS I(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, cur_lhs_fft, &freq_hi, 4);
      
      /*****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of LHS I(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, cur_lhs_fft+freq_hi, &freq_hi, 4);

      /*****************************************/

      XGSet2D("linlog", "Freq", "Mag of LHS P(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, pow_lhs_fft, &freq_hi, 4);
      
      /*****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of LHS P(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, pow_lhs_fft+freq_hi, &freq_hi, 4);
    }
    /******************************************/
    
    if(rhs_flag) {
      XGSet2D("linlin", "Time", "Local RHS V(t)", "closed",
	      100, 300, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, phi_rhs_hist, &thist_hi, 2);
      
      /****************************************/
      
      XGSet2D("linlin", "Time", "Local RHS I(t)", "closed",
	      100, 300, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, cur_rhs_hist, &thist_hi, 2);
      
      /****************************************/
      
      XGSet2D("linlin", "Time", "Local RHS P(t)", "closed",
	      100, 300, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(Local_t_array, pow_rhs_hist, &thist_hi, 2);

      /****************************************/

      XGSet2D("linlog", "Freq", "Mag of RHS V(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, phi_rhs_fft, &freq_hi, 4);
      
      /****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of RHS V(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, phi_rhs_fft+freq_hi, &freq_hi, 4);

      /****************************************/

      XGSet2D("linlog", "Freq", "Mag of RHS I(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, cur_rhs_fft, &freq_hi, 4);
      
      /****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of RHS I(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, cur_rhs_fft+freq_hi, &freq_hi, 4);

      /****************************************/

      XGSet2D("linlog", "Freq", "Mag of RHS P(f)", "closed",
	      100, 250, 1.0, 1.0, True, True, 0.0, 0.0, 0.0, 0.0);
      
      XGCurve(f_array, pow_rhs_fft, &freq_hi, 4);
      
      /****************************************/
      
      XGSet2D("linlin", "Freq", "Phase of RHS P(f)", "closed",
	      400, 250, 1.0, 1.0, True, False, 1.0, 1.0, -180.0, 180.0);
      
      XGCurve(f_array, pow_rhs_fft+freq_hi, &freq_hi, 4);
    }
  }
  
  /********************************************/
  
  if(nsp && ecollisional) {
    XGSet3D("linlinlin", "X", "Y", "Profile Ionization(x,y)",
	    45.0, -120.0, "closed", 200, 150, 1.0, 1.0, 1.0,
	    False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
    XGSurf(x_array, y_array, ionrate, &ngx, &ngy, 3);
    
    /****************************************/
    XGSet3D("linlinlin", "X", "Y", "Profile Excitation(x,y)",
	    45.0, -120.0, "closed", 200, 150, 1.0, 1.0, 1.0,
	    False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
    XGSurf(x_array, y_array, extrate, &ngx, &ngy, 3);
  }

  /********************************************/

  if(nsp && icollisional) {
    XGSet3D("linlinlin", "X", "Y", "Profile Charge X(x,y)",
	    45.0, -120.0, "closed", 200, 150, 1.0, 1.0, 1.0,
	    False, False, True, 0.0, xlength, 0.0, ylength, 0.0, 0.0);
  
    XGSurf(x_array, y_array, chrgxrate, &ngx, &ngy, 3);
  }
  
  /********************************************/  
} 

/***************************************************************/
/* Dumping the current state of the system into a binary file. */
/* Note: the binary file is written in the format used by IBM  */
/* and DEC compatible machines (low bytes followed by high     */
/* bytes). For this Dump() and Restore() call the functions    */
/* myfread() and myfwrite().                                   */

char Revision[]={'1','.','0','1'};

void Dump(char *filename)
{
  register int i, j, k, isp;
  SCALAR ftemp;
  char dropfile[20];
  FILE *DMPFile, *DropFile;
  
  if ((DMPFile = fopen(filename, "w")) == NULL) {
    puts("Dump: open failed");
    exit(-1);
  }
  
  XGWrite(Revision, 1, 4, DMPFile, "char");
  ftemp = t;
  XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&lhs_extq_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&lhs_sigma_total_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&rhs_extq_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&rhs_sigma_total_1, 4, 1, DMPFile, SCALAR_CHAR);
  XGWrite(&nsp, 4, 1, DMPFile, "int");
  XGWrite(np, 4, nsp, DMPFile, "int");
  for (isp=0; isp<nsp; isp++) {
    if(np[isp]>0){
      for (i=0; i<np[isp]; i++) {
	ftemp = x[isp][i]/fncx;
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	ftemp = y[isp][i]/fncy;
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	ftemp = vxscale[isp]*vx[isp][i];
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	ftemp = vyscale[isp]*vy[isp][i];
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	ftemp = vzscale[isp]*vz[isp][i];
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      }
    }
  }
  XGWrite(&ncx, 4, 1, DMPFile, "int");
  XGWrite(&ncy, 4, 1, DMPFile, "int");
  for (isp=0; isp<nsp; isp++) {
    for(i=0; i<=ncx; i++) {
      for(j=0; j<=ncy; j++) {
	ftemp = sp_sigma[isp][i][j];
	XGWrite(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
      }
    }
  }
  
  /**** Wall profile diagnostics ****/
  if(wall_flag) {
    for(k=0; k<SIDES; k++) {
      for(isp=0; isp<nsp; isp++) {
	XGWrite(wall_q[k][isp],  4, ngy, DMPFile, SCALAR_CHAR);
	XGWrite(th_ave1[k][isp], 4, ngy, DMPFile, SCALAR_CHAR);
	XGWrite(th_ave2[k][isp], 4, ngy, DMPFile, SCALAR_CHAR);
	
	for (j=0; j<ngy; j++) {
	  XGWrite(fe[k][isp][j],  4, nbin[k][isp], DMPFile, SCALAR_CHAR);
	  XGWrite(fth[k][isp][j], 4, nbin[k][isp], DMPFile, SCALAR_CHAR);
	}
      }
    }
  }
  fclose(DMPFile);

  /***  Then write the number of particles in ascii  *****/
  strcpy(dropfile, filename);
  strcat(dropfile, ".drp");
  
  DropFile = fopen(dropfile, "a");
  fprintf(DropFile, "%lf ", t);
  for(i=0; i<nsp; i++) fprintf(DropFile, "%d ", np[i]);
  fprintf(DropFile, "\n");
  fclose(DropFile);
}

/****************************************************************/

void Restore(char *filename)
{
  register int i, j, k, isp;
  int tmp_ncx, tmp_ncy, read_nsp, read_np[NSMAX];
  char Rev[5];
  SCALAR ftemp;
  FILE *DMPFile;
  
  puts("Start reading the dumpfile");
  
  if ((DMPFile = fopen(filename, "r+b")) == NULL) {
    puts("Restore: open failed");
    return;
  }
  XGRead(Rev, 1, 4, DMPFile, "char");
  for (i=0; i<4; i++)
    if (Rev[i]!=Revision[i]) {
      puts("Restore: Incompatible dump file version");
      exit(-1);
    } 
  
  XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
  t = ftemp;
  XGRead(&lhs_extq, 4, 1, DMPFile, SCALAR_CHAR); 
  XGRead(&lhs_sigma_total, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&rhs_extq, 4, 1, DMPFile, SCALAR_CHAR);       
  XGRead(&rhs_sigma_total, 4, 1, DMPFile, SCALAR_CHAR);
  XGRead(&read_nsp, 4, 1, DMPFile, "int");
  if(read_nsp>nsp) {
    puts("Restore: Dump file contains more species than input file");
    exit(-1);
  }
  XGRead(read_np, 4, read_nsp, DMPFile, "int");
  for (isp=0; isp<read_nsp; isp++) {
    if(read_np[isp]>0){
      np[isp] = read_np[isp];
      for (i=0; i<np[isp]; i++) {
	XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	x[isp][i] = ftemp*fncx;
	XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	y[isp][i] = ftemp*fncy;
	XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	vx[isp][i]= ftemp/vxscale[isp];
	XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	vy[isp][i]= ftemp/vyscale[isp];
	XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	vz[isp][i]= ftemp/vzscale[isp];
      }
    }
  }
  XGRead(&tmp_ncx, 4, 1, DMPFile, "int");
  XGRead(&tmp_ncy, 4, 1, DMPFile, "int");
  if(tmp_ncx == ncx && tmp_ncy == ncy) {
    for (isp=0; isp<read_nsp; isp++) {
      for(i=0; i<=ncx; i++) {
	for(j=0; j<=ncy; j++) {
	  XGRead(&ftemp, 4, 1, DMPFile, SCALAR_CHAR);
	  sp_sigma[isp][i][j] = ftemp;
	}
      }
    }
  }
  /**** Read Wall profile diagnostics ****/
  if(wall_flag && wall_read_flag) {
    puts("reading wall diag. from the dumpfile");
    for(k=0; k<SIDES; k++) {
      for(isp=0; isp<read_nsp; isp++) {
	XGRead(wall_q[k][isp],  4, ngy, DMPFile, SCALAR_CHAR);
	XGRead(th_ave1[k][isp], 4, ngy, DMPFile, SCALAR_CHAR);
	XGRead(th_ave2[k][isp], 4, ngy, DMPFile, SCALAR_CHAR);
	for (j=0; j<ngy; j++) {
	  XGRead(fe[k][isp][j],  4, nbin[k][isp], DMPFile, SCALAR_CHAR);
	  XGRead(fth[k][isp][j], 4, nbin[k][isp], DMPFile, SCALAR_CHAR);
	}
      }
    }
  }
  puts("finished reading the dumpfile");
  fclose(DMPFile);
}

/****************************************************************/

void Quit()
{
  
}

/****************************************************************/
