#include "pdp2.h"

/***************************************************************/

void load(int isp, SCALAR initn, int loader, int fill_region, SCALAR xleft, SCALAR xright,
	  SCALAR ylow, SCALAR yhigh, SCALAR v0xi, SCALAR v0yi, SCALAR v0zi, SCALAR vti)
{
  static int i=0;
  register int n, nx, ny;
  SCALAR fnp, x0, xlen, y0, ylen;
  
  if(fill_region) {
    x0  = 0.0;
    xlen= fncx;
    y0  = 0.0;
    ylen= fncy;
  }
  else {
    x0  = fncx*xleft/xlength;
    xlen= fncx*(xright -xleft)/xlength;
    y0  = fncy*ylow/ylength;
    ylen= fncy*(yhigh -ylow)/ylength;
  }
  
  np[isp]= initn*xlen*ylen*dx*dy*zlength/nc2p/weight[isp];
  if (np[isp] > maxnp[isp]) {
    printf("LOAD: too many particles, species %d\n", isp);
    exit(1);
  }
  
  /******************************************************/
  /* Loading the velocities and positions depending on
     the given flag.  This Loader can load particles
     using a quiet start method, uniform XY or randomly */
  
  switch (loader) {
  case QUIET_START:
    for(n=0; n<np[isp]; n++) {
      vx[isp][n] = vy[isp][n] = vz[isp][n] = 0.0;
      if (vti>0.0)
	maxwellv(&vx[isp][n], &vy[isp][n], &vz[isp][n], vti,
		 frand(), revers(n+i,2), revers(n+i,7));
      
      vx[isp][n] =(vx[isp][n] +v0xi)/vxscale[isp];
      vy[isp][n] =(vy[isp][n] +v0yi)/vyscale[isp];
      vz[isp][n] =(vz[isp][n] +v0zi)/vzscale[isp];
      
      x[isp][n]= x0 +xlen*revers(n+i, 5);
      y[isp][n]= y0 +ylen*revers(n+i, 3);
    }
    break;
  case UNIFORM_XY:
    fnp= np[isp];
    nx = sqrt(fnp)/fncy+.5;
    ny = sqrt(fnp)/fncx+.5;
    np[isp]= nx*ny*fncx*fncy;
    
    for(n=0; n<np[isp]; n++) {
      vx[isp][n] = vy[isp][n] = vz[isp][n] = 0.0;
      if (vti>0.0)
	maxwellv(&vx[isp][n], &vy[isp][n], &vz[isp][n], vti,
		 revers(n+i,2), revers(n+i,7), revers(n+i,3));
      
      vx[isp][n] =(vx[isp][n] +v0xi)/vxscale[isp];
      vy[isp][n] =(vy[isp][n] +v0yi)/vyscale[isp];
      vz[isp][n] =(vz[isp][n] +v0zi)/vzscale[isp];
      
      x[isp][n]= (n%(ncx*nx) +.5)/nx;
      y[isp][n]= (n/(ncx*nx) +.5)/ny;
    }
    break;
  case RANDOM:
    for(n=0; n<np[isp]; n++) {
      vx[isp][n] = vy[isp][n] = vz[isp][n] = 0.0;
      if (vti>0.0)
	maxwellv(&vx[isp][n], &vy[isp][n], &vz[isp][n], vti,
		 frand(), frand(), frand());
      
      vx[isp][n] =(vx[isp][n] +v0xi)/vxscale[isp];
      vy[isp][n] =(vy[isp][n] +v0yi)/vyscale[isp];
      vz[isp][n] =(vz[isp][n] +v0zi)/vzscale[isp];
      
      x[isp][n]= x0 +xlen*frand();
      y[isp][n]= y0 +ylen*frand();
    }
    break;
  default:
    puts("LOAD: Bad value for loader flag");
    exit(-1);
    break;
  }
  
  /***************************************************/
  /* if one particle, stick it in center and give it */
  /* drift + random thermal part                     */
  if (np[isp]==1) {
    x[isp][0]=x0 +xlen/2.0;
    y[isp][0]=y0 +ylen/2.0;
    vx[isp][0]=(v0xi+2.0*(frand()-0.5)*vti)/vxscale[isp];
    vy[isp][0]=(v0yi+2.0*(frand()-0.5)*vti)/vyscale[isp];
    vz[isp][0]=(v0zi+2.0*(frand()-0.5)*vti)/vzscale[isp];
  }
  i+= np[isp];
}

/***************************************************************/
