#include "pdp2.h"
#include "xgrafix.h"

void main(int argc, char *argv[])
{
  display_title();        /* Display XPDP2 title           */
  XGInit(argc, argv, &t); /* Initialize XGrafix stuff      */
  
  start();                /* Allocate arrays and initailze */
  setrho();               /* Set initial charge density    */
  (*fields_ptr)();        /* Initialize field arrays       */
  if(theRunWithXFlag) {
    history();
    InitWindows();        /* Initialize diagnostic windows */
  }
  XGStart();              /* Start XGrafix main loop       */
}

/***********************************************************/
/*  The main physics loop                                  */

void XGMainLoop()
{
  register int i, j, n, isp;
  SCALAR frac;
  
  t += dt;
  for(isp=0; isp<nsp; isp++) {
    if(!(k_count[isp]%sp_k[isp])) {
      it[isp]++;
      (*move_ptr)(isp);
      boundary(isp);
      if(wall_flag) wall_diagnos(isp);
      mcc(isp);
      gather(isp);
      k_count[isp]=0;
    }
    
    k_count[isp]++;
    frac = ((SCALAR)k_count[isp])/sp_k[isp];
    for(i=0; i<=ncx; i++)
      for(j=0; j<=ncy; j++)
	sp_n[isp][i][j]= (1- frac)*sp_n_0[isp][i][j]
	  +frac*sp_n_k[isp][i][j] +sp_n_mcc[isp][i][j];
    
    /**** Digitally smooth the species densities in x and y. */
    for (n=0; n<sflag; n++) {
      if(ybc_flag == PERIODIC){
	Periodic_Smooth(sp_n[isp],   ncx, ncy);
	Periodic_Smooth(sp_vx0[isp], ncx, ncy);
	Periodic_Smooth(sp_vy0[isp], ncx, ncy);
	Periodic_Smooth(sp_vz0[isp], ncx, ncy);
	Periodic_Smooth(sp_vt[isp],  ncx, ncy);
      }
      else { 
	TWOD_One_2_One(sp_n[isp],   ncx, ncy);
	TWOD_One_2_One(sp_vx0[isp], ncx, ncy);
	TWOD_One_2_One(sp_vy0[isp], ncx, ncy);
	TWOD_One_2_One(sp_vz0[isp], ncx, ncy);
	TWOD_One_2_One(sp_vt[isp],  ncx, ncy);
      }
    }
  }
  
  /**************************************************/
  /** Now do the field solve and finally, calculate */
  /** the species E field  */
  
  (*fields_ptr)();
  for(isp=0; isp<nsp; isp++)
    for(i=0; i <= ncx; i++)
      for(j=0; j <= ncy; j++) {
        sp_ex[isp][i][j] += ex[i][j];
        sp_ey[isp][i][j] += ey[i][j];
      }
  
  if(theRunWithXFlag) history();
}

/***************************************************************/

void display_title()
{
  puts("\nXPDP2 Version 2.3");
  puts("Copyright (C) 1991-1998");
  puts("Regents of the University of California");
  puts("Plasma Theory and Simulation Group");
  puts("University of California\n");
}

/***************************************************************/

