#include "pdp2.h"
#include "xgrafix.h"
#include <string.h>
FILE *InputDeck;

/***********************************************************/

void start()
{
  register int i, j, k, n, isp, ix, iy;
  SCALAR temp, ttmag;
  char aachar[80], c;
  
  /***********************************************/
  /* Open files and read in "general" parameters */
  
  if (!WasInputFileGiven) {
    puts("Syntax:");
    puts("\tPDP2 -i <inputdeck> -d [dumpfile]");
    exit(1);
  }
  InputDeck = fopen(theInputFile, "r");
  
  /*********************/
  /*   DEFAULTS        */
  
  wall_flag      = 0;
  wall_read_flag = 0;
  ecollisional   = 0;
  icollisional   = 0;
  ionsp          = 0;
  epsilon        = 1.0;
  ybc_flag       = DIRICHLET;
  
  /* Read lines until we get to numbers */
  while (fscanf(InputDeck,"%d %d %d %lf %lf %lf %lf %lf %lf", &nsp, &ncx, &ncy,
		&nc2p, &dt, &xlength, &ylength, &zlength, &epsilon) <9)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %lf %lf %lf %lf %lf %lf %d", &lhs_flag, &lhs_length,
		&lhs_dc, &lhs_ac, &lhs_w0, &lhs_theta0, &lhs_extc, &lhs_gap) <8)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %lf %lf %lf %lf %lf %lf %d", &rhs_flag, &rhs_length,
		&rhs_dc, &rhs_ac, &rhs_w0, &rhs_theta0, &rhs_extc, &rhs_gap) <8)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %lf %lf %d %d %lf %d", &rhoback, &bmag,
		&btheta, &bphi, &nfft, &sflag, &tol_pois, &nstrcmax) <8)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %d %d %lf %lf", &ecollisional, &icollisional,
		&ionsp, &pressure, &gtemp) <5)
    fscanf(InputDeck, "%s", aachar);

  while (fscanf(InputDeck, "%d %lf %d %d", &ybc_flag,&temp,&wall_flag,&wall_read_flag) <4)
    fscanf(InputDeck, "%s", aachar);

  /*****************************************/
  
  btheta     *= 2*M_PI/360.0;
  bphi       *= 2*M_PI/360.0;
  
  lhs_theta0 *= 2*M_PI/360.0;
  rhs_theta0 *= 2*M_PI/360.0;
  lhs_w0     *= 2*M_PI;
  rhs_w0     *= 2*M_PI;
  epsilon    *= EPS0;
  
  lhs_extq    = 0.0;
  rhs_extq    = 0.0;
   seed        = (long) getpid(); /* This is the seed used in the random number generator */
 /* seed=31207321; */
  
  ngx = ncx+1;  
  ngy = ncy+1;
  fncx= ncx; 
  fncy= ncy;
  
  dx = xlength/fncx; 
  dy = ylength/fncy; 

  /*****************************************/
  /*****   Fix field parameters     ********/
  
  fields_ptr= fields;
  del_area  = zlength*dy;

  lhs_ny = lhs_length*ncy/ylength;   /* Left hand side  */
  if(!(ncy%2) && lhs_ny%2) lhs_ny++;
  if(ncy%2 && !(lhs_ny%2)) lhs_ny++;
  lhs_fnys = lhs_nys = (ncy-lhs_ny)/2;
  lhs_fnyf = lhs_nyf = ncy-(ncy-lhs_ny)/2;
  if(lhs_gap==0 && lhs_length != ylength) lhs_gap=1;
  
  rhs_ny = rhs_length*ncy/ylength;   /* Right hand side */
  if(!(ncy%2) && rhs_ny%2) rhs_ny++;
  if(ncy%2 && !(rhs_ny%2)) rhs_ny++;
  rhs_fnys = rhs_nys = (ncy-rhs_ny)/2;
  rhs_fnyf = rhs_nyf = ncy-(ncy-rhs_ny)/2;
  if(rhs_gap==0 && rhs_length != ylength) rhs_gap=1;
  
  /******* If y is periodic  ********/
  if(ybc_flag == PERIODIC) {
    puts("Periodic in y.\n");
    fields_ptr= fields2;
    del_area  = ylength*zlength;
    
    rhs_flag=rhs_gap=0;       /* set rhs to ground */
    rhs_length=ylength;
    rhs_dc=rhs_ac=rhs_w0=rhs_theta0=rhs_extc=0.0;
    rhs_fnyf = rhs_nyf= ncy;
    rhs_fnys = rhs_nys= 0;
    
    nstrcmax=0;
    lhs_extq_2=lhs_extq_3=0.0;
    
    lhs_length=ylength;       /* adjust lhs */
    lhs_gap=0;
    lhs_fnyf = lhs_nyf= ncy;
    lhs_fnys = lhs_nys= 0;
    lhs_extl=lhs_extr = 0.0;

    /**  In the periodic case ncy= 2^n **/
    for(i=0; i<= 15; i++)
      if(ncy == (1<<i)) {
        i=0;  break;
      }
    if(i) {
      puts("START: ncy is not an integer power of 2");
      exit(1);
    }
  }
  
  /******************************************************/
  /* Check for errors in the "general" input parameters */
  
  move_ptr = move;
  if(bmag > 0.0) {
    move_ptr = mag_move;
  }
  /*
  else if(nstrcmax>0)
    move_ptr = move_internal;
  */
  if (ncx<2 || ncy<2) {
    puts("START: dont be cute with ncx,ncy<2\n");
    exit(1);
  }
  if (lhs_length>ylength || rhs_length>ylength) {
    puts("START: dont be cute with electrode length > y length\n");
    exit(1);
  }
  /**end djc**/
  if (dt <= 0.0) {
    puts("START: dt <= 0!!\n");
    exit(1);
  }
  
  /* Check to see if nfft is an integer power of 2 */
  if(nfft) {
    for(i=0; i<= 15; i++)
      if(nfft == (1<<i)) {
        i=0;  break;
      }
    if(i) {
      puts("START: nfft is not an integer power of 2");
      exit(1);
    }
  }
  
  interval=1;
  
  /***************************************/
  /* Allocate space for field parameters */
  
  x_array= (SCALAR *)malloc(ngx*sizeof(SCALAR));
  for(i=0; i<=ncx; i++) x_array[i]= i*dx;
  
  y_array= (SCALAR *)malloc(ngy*sizeof(SCALAR));
  for(i=0; i<=ncy; i++) y_array[i]= i*dy;

  /***********  Species Stuff  ********/
  
  sp_n    = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_n_0  = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_n_k  = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_n_mcc= (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_vx0  = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_vy0  = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_vz0  = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_vt   = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_ex   = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_ey   = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  sp_sigma= (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
  
  for(isp=0; isp<nsp; isp++) {
    sp_n[isp]    = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_n_0[isp]  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_n_k[isp]  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_n_mcc[isp]= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_vx0[isp]  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_vy0[isp]  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_vz0[isp]  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_vt[isp]   = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_ex[isp]   = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_ey[isp]   = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    sp_sigma[isp]= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    for(i=0; i<=ncx; i++) {
      sp_n[isp][i]    = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_n_0[isp][i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_n_k[isp][i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_n_mcc[isp][i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_vx0[isp][i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_vy0[isp][i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_vz0[isp][i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_vt[isp][i]   = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_ex[isp][i]   = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_ey[isp][i]   = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      sp_sigma[isp][i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
      for(j=0; j<=ncy; j++) {
	sp_n[isp][i][j]    = sp_n_0[isp][i][j]= sp_n_k[isp][i][j]= 0.0;
	sp_n_mcc[isp][i][j]= sp_vx0[isp][i][j]= sp_vy0[isp][i][j]= 0.0;
	sp_vz0[isp][i][j]= sp_vt[isp][i][j]= 0.0;
	sp_ex[isp][i][j] = sp_ey[isp][i][j] = sp_sigma[isp][i][j]= 0.0;
      }
    }
  }
  
  /***********  More field parameters  ********/
  
  rho     = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  source  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  phi     = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  phi_pois= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  phi_intl= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  ex      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  ey      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  ax      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  ay      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  sigma   = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  eps_array=(SCALAR **)malloc(ngx*sizeof(SCALAR *));
  for(i=0; i<=ncx; i++) {
    rho[i]     = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    source[i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    phi[i]     = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    phi_pois[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
    phi_intl[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
    ex[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    ey[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    ax[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    ay[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    sigma[i]   = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    eps_array[i]=(SCALAR *)malloc(ngy*sizeof(SCALAR));
    
    for(j=0; j<=ncy; j++) {
      phi[i][j] = phi_pois[i][j]= phi_intl[i][j]= sigma[i][j] = 0.0;
      eps_array[i][j]= 1.0;
    }
  }
  
  /**djc**new**/
  if(ybc_flag == PERIODIC) {
    khi=ncy/2;
    khi_plus1=ncy/2 +1;
    rhok      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    phik      = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    show_phik = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    show_rhok = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    for(i=0; i<=ncx; i++) {
      rhok[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      phik[i]      = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      show_phik[i] = (SCALAR *)malloc(khi_plus1*sizeof(SCALAR));
      show_rhok[i] = (SCALAR *)malloc(khi_plus1*sizeof(SCALAR));
	for(j=0;j<khi_plus1;j++) show_rhok[i][j] = show_phik[i][j] = 0.0;
    }
    
    dk = (SCALAR  *)malloc(ngy*sizeof(SCALAR));
    for (k=0; k< khi; k++) 
      dk[2*k] = dk[2*k+1] = 1.0 +2.0*(dx/dy*sin(M_PI*k/ncy))*(dx/dy*sin(M_PI*k/ncy));
    dk[1]=1.0 +2.0*(dx/dy)*(dx/dy);
    
    k_array = (SCALAR  *)malloc(khi_plus1*sizeof(SCALAR));
    for (k=0; k<=khi; k++) k_array[k]= (SCALAR) k;
    
    phi_mid_k_t = (SCALAR **)malloc(khi_plus1*sizeof(SCALAR *));
    phi_mid_k_f = (SCALAR **)malloc(khi_plus1*sizeof(SCALAR *));
    for(i=0;i<khi_plus1;i++) {
      phi_mid_k_t[i] = (SCALAR *)malloc(2*nfft*sizeof(SCALAR));
      phi_mid_k_f[i] = (SCALAR *)malloc(2*nfft*sizeof(SCALAR));
      for(j=0;j<2*nfft;j++) phi_mid_k_f[i][j] = 0.0;
    }
  }
  /**end djc**/
  
  if(lhs_flag) {
    phi_lap_lhs= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    for(i=0; i<=ncx; i++) {
      phi_lap_lhs[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
      for(j=0; j<=ncy; j++) phi_lap_lhs[i][j]= 0.0;
    }
  }
  if(rhs_flag) {
    phi_lap_rhs= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    for(i=0; i<=ncx; i++) {
      phi_lap_rhs[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
      for(j=0; j<=ncy; j++) phi_lap_rhs[i][j]= 0.0;
    }
  }
  
  /*********** fft diagnostics      ***********/
  
  if(theRunWithXFlag) {
    if(nfft) {
      freq_hi= nfft/2;
      df= 1/(nfft*dt);
      f_array= (SCALAR *)malloc(nfft*sizeof(SCALAR));
      for(i=0; i< nfft; i++) f_array[i]= i*df;
      
      Local_t_array = (SCALAR *) malloc(nfft*sizeof(SCALAR));
      phi_mid_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
      phi_mid_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
      for(i=0; i< nfft; i++) phi_mid_fft[i] = 0.0;
      
      if(lhs_flag) {
	phi_lhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	cur_lhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	pow_lhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	
	phi_lhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	cur_lhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	pow_lhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	
	for(i=0; i< nfft; i++) 
	  pow_lhs_fft[i] = cur_lhs_fft[i] = phi_lhs_fft[i] = 0.0;
      }
      if(rhs_flag) {
	phi_rhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	cur_rhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	pow_rhs_hist= (SCALAR *)malloc(nfft*sizeof(SCALAR));
	
	phi_rhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	cur_rhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	pow_rhs_fft = (SCALAR *)malloc(nfft*sizeof(SCALAR));
	
	for(i=0; i< nfft; i++) 
	  pow_rhs_fft[i] = cur_rhs_fft[i] = phi_rhs_fft[i] = 0.0;
      }
      
      /**** Time averaged arrays which only show up when nfft is set ****/
      sp_n_ave= (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
      sp_n_ave_show= (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
      for(isp=0; isp<nsp; isp++) {
	sp_n_ave[isp]= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
	sp_n_ave_show[isp]= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
	for(i=0; i<=ncx; i++) {
	  sp_n_ave[isp][i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
	  sp_n_ave_show[isp][i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
	  for(j=0; j<=ncy; j++) sp_n_ave[isp][i][j]=0.0;
	}
      }
      
      phi_ave= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
      phi_ave_show= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
      for(i=0; i<=ncx; i++) {
	phi_ave[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
	phi_ave_show[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
	for(j=0; j<=ncy; j++) phi_ave[i][j]=0.0;
      }
    }
    
    /******** The MCC diagnostics   *********/
    
    extrate  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    ionrate  = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    chrgxrate= (SCALAR **)malloc(ngx*sizeof(SCALAR *));
    for(i=0; i<=ncx; i++) {
      extrate[i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      ionrate[i]  = (SCALAR *)malloc(ngy*sizeof(SCALAR));
      chrgxrate[i]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
      for(j=0; j<=ncy; j++) 
	extrate[i][j]= ionrate[i][j]= chrgxrate[i][j]= 0.0;
    }
    
    /*********** Time diagnostics       *********/
    if(lhs_flag) {
      com_phc_lhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_phs_lhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_phi_lhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_cur_lhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_pow_lhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    }
    if(rhs_flag) {
      com_phc_rhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_phs_rhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_phi_rhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_cur_rhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      com_pow_rhs_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    }
    com_phi_mid_hist = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    t_array = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    np_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    kes_hist= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    
    for (i=0; i<nsp; i++) {
      np_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      kes_hist[i]= (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    }
  }
  
  /****************************/
  /* Allocate particle arrays */
  
  x = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  y = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vx= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vy= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vz= (SCALAR **)malloc(nsp*sizeof(SCALAR *));

  /********************************************/
  /* Set up the parameters for each "species" */

  for (isp=0; isp<nsp; ++isp) {
    species(isp);
    
    Exscale[isp] = 0.5*vxscale[isp]*vxscale[isp]*m[isp]/(1.602e-19);
    Eyscale[isp] = 0.5*vyscale[isp]*vyscale[isp]*m[isp]/(1.602e-19);
    Ezscale[isp] = 0.5*vzscale[isp]*vzscale[isp]*m[isp]/(1.602e-19);
    
    if(bmag > 0.0) {
      ttmag=tan(0.5*sp_k[isp]*dt*qm[isp]*bmag);
      ttx[isp]=ttmag*sin(btheta)*cos(bphi);
      tty[isp]=ttmag*sin(btheta)*sin(bphi);
      ttz[isp]=ttmag*cos(btheta);
      ssx[isp]=2.0*ttx[isp]/(1.0+ttmag*ttmag);
      ssy[isp]=2.0*tty[isp]/(1.0+ttmag*ttmag);
      ssz[isp]=2.0*ttz[isp]/(1.0+ttmag*ttmag);
    }
  }
  
  /*********************************************/
  /*************  Reading structures  **********/
  
  nstrctype=0;
  strc = (IntStructType *)malloc(nstrcmax*sizeof(IntStructType));
  for(k=0; k<nstrcmax; k++) {
    while (fscanf(InputDeck,"%d %d %d %d %lf %lf %s",
		  &strc[k].xl, &strc[k].yl, &strc[k].xr, &strc[k].yu,
		  &strc[k].matrl, &strc[k].volt, strc[k].name) <7)
      fscanf(InputDeck, "%s", aachar);
    
    if(k==0){
      strcpy(strc_name[nstrctype], strc[k].name);
      strc[k].type=nstrctype;
      nstrctype=1;
    }
    else {
      for(j=0; j<nstrctype; j++) {
	if(!strcmp(strc_name[j], strc[k].name)) break;
      }
      if(j<nstrctype) strc[k].type = j;
      else {
	strcpy(strc_name[nstrctype], strc[k].name);
	strc[k].type=nstrctype;
	nstrctype++;
      }
    }
  }
  fclose(InputDeck);
  
  stc_np_hist = (SCALAR ***)malloc(nstrctype*sizeof(SCALAR **));
  for(j=0; j<nstrctype; j++){
    stc_np_hist[j] = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    for (i=0; i<nsp; i++) {
      stc_np_hist[j][i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    }
  }
  
  grid_mask= (int **)malloc(ngx*sizeof(int *));
  cell_mask= (int **)malloc(ngx*sizeof(int *));
  cell_k   = (int **)malloc(ngx*sizeof(int *));
  conductor= (int **)malloc(ngx*sizeof(int *));
  face     = (int **)malloc(ngx*sizeof(int *));
  area     = (SCALAR **)malloc(ngx*sizeof(SCALAR *));
  for(i=0; i<=ncx; i++) {
    grid_mask[i]= (int *)malloc(ngy*sizeof(int));
    cell_mask[i]= (int *)malloc(ngy*sizeof(int));
    cell_k[i]   = (int *)malloc(ngy*sizeof(int));
    conductor[i]= (int *)malloc(ngy*sizeof(int));
    face[i]     = (int *)malloc(ngy*sizeof(int));
    area[i]     = (SCALAR *)malloc(ngy*sizeof(SCALAR));
    for(j=0; j<=ncy; j++) {
      grid_mask[i][j]= 0;          /* No structure */
      cell_mask[i][j]= 0;          /* No structure */
      cell_k[i][j]   = 0;          /* No structure */
      conductor[i][j]= 1;          /* No conductor */
      face[i][j]     = NO_FACE;    /* No face      */
      area[i][j]     = dy*zlength; /* surface area */
    }
  }
  
  /****************************************/
  /* Setting cell flags due to structures */
  
  for(i=0; i<= ncx; i++)
    for(j=0; j<= ncy; j++)
      for(k=0; k<nstrcmax; k++) {
	if(j>= strc[k].yl && j < strc[k].yu && i>= strc[k].xl && i < strc[k].xr) {
	  cell_mask[i][j]= 1;
	  cell_k[i][j]   = k;
	  eps_array[i][j]= strc[k].matrl;
	}
      }
  
  for(i=0; i< ncx; i++)
    for(j=0; j< ncy; j++) {
      if(cell_mask[i][j]) {
	grid_mask[i][j] = grid_mask[i+1][j] = grid_mask[i][j+1] = grid_mask[i+1][j+1] = 1;
	if(strc[cell_k[i][j]].matrl == 0.0) {
	  conductor[i][j]= conductor[i+1][j]= conductor[i][j+1]= conductor[i+1][j+1]= 0;
	  phi_intl[i][j] = phi_intl[i+1][j] = phi_intl[i][j+1] = phi_intl[i+1][j+1] = strc[cell_k[i][j]].volt;
	}
      }
    }
  
  for(j=0; j<=ncy; j++) eps_array[ncx][j] = eps_array[ncx-1][j];
  for(i=0; i<=ncx; i++) eps_array[i][ncy] = eps_array[i][ncy-1];
  
  for(i=1; i< ncx; i++)
    for(j=1; j< ncy; j++) {
      if (grid_mask[i][j]) {
	if((eps_array[i-1][j]==1.0 && eps_array[i][j]==1.0 && eps_array[i-1][j-1]==1.0) ||
	   (eps_array[i-1][j]!=0.0 && eps_array[i][j]!=0.0 && eps_array[i-1][j-1]!=0.0 && eps_array[i][j-1]==0.0)) {
	  face[i][j]=UL_CORN;
	  area[i][j]= 0.5*(dx+dy)*zlength;
	}
	else if((eps_array[i-1][j]==1.0 && eps_array[i][j]==1.0 && eps_array[i][j-1]==1.0) ||
		(eps_array[i-1][j]!=0.0 && eps_array[i][j]!=0.0 && eps_array[i][j-1]!=0.0 && eps_array[i-1][j-1]==0.0)) {
	  face[i][j]=UR_CORN;
	  area[i][j]= 0.5*(dx+dy)*zlength;
	}
	else if((eps_array[i-1][j]==1.0 && eps_array[i][j]==1.0 && eps_array[i-1][j-1]!=1.0 && eps_array[i][j-1]!=1.0) ||
		(eps_array[i-1][j]!=0.0 && eps_array[i][j]!=0.0 && eps_array[i-1][j-1]==0.0 && eps_array[i][j-1]==0.0)) {
	  face[i][j]=UP;
	  area[i][j]= dx*zlength;
	}
	else if((eps_array[i-1][j-1]==1.0 && eps_array[i][j-1]==1.0 && eps_array[i-1][j]==1.0) ||
		(eps_array[i-1][j]!=0.0 && eps_array[i-1][j-1]!=0.0 && eps_array[i][j-1]!=0.0 && eps_array[i][j]==0.0)) {
	  face[i][j]=LL_CORN;
	  area[i][j]= 0.5*(dx+dy)*zlength;
	}
	else if((eps_array[i-1][j-1]==1.0 && eps_array[i][j-1]==1.0 && eps_array[i][j]==1.0) ||
		(eps_array[i][j]!=0.0 && eps_array[i-1][j-1]!=0.0 && eps_array[i][j-1]!=0.0 && eps_array[i-1][j]==0.0)) {
	  face[i][j]=LR_CORN;
	  area[i][j]= 0.5*(dx+dy)*zlength;
	}
	else if((eps_array[i-1][j-1]==1.0 && eps_array[i][j-1]==1.0 && eps_array[i][j]!=1.0 && eps_array[i-1][j]!=1.0) ||
		(eps_array[i-1][j-1]!=0.0 && eps_array[i][j-1]!=0.0 && eps_array[i][j]==0.0 && eps_array[i-1][j]==0.0)) {
	  face[i][j]=DOWN;
	  area[i][j]= dx*zlength;
	}
	else if((eps_array[i-1][j-1]==1.0 && eps_array[i-1][j]==1.0 && eps_array[i][j]!=1.0 && eps_array[i][j-1]!=1.0) ||
		(eps_array[i-1][j-1]!=0.0 && eps_array[i-1][j]!=0.0 && eps_array[i][j]==0.0 && eps_array[i][j-1]==0.0)) {
	  face[i][j]=LEFT;
	  area[i][j]= dy*zlength;
	}
	else if((eps_array[i-1][j-1]!=1.0 && eps_array[i-1][j]!=1.0 && eps_array[i][j]==1.0 && eps_array[i][j-1]==1.0) ||
		(eps_array[i-1][j-1]==0.0 && eps_array[i-1][j]==0.0 && eps_array[i][j]!=0.0 && eps_array[i][j-1]!=0.0)) {
	  face[i][j]=RIGHT;
	  area[i][j]= dy*zlength;
	}
      }
    }
  /*********************************************/
  /* Fixing the face flags for left and right  */
  
  for(j=0; j<=ncy; j++) {
    conductor[0][j] = 0;    face[0][j]  = RIGHT; area[0][j]  = dy*zlength;
    conductor[ncx][j]= 0;   face[ncx][j]= LEFT;  area[ncx][j]= dy*zlength;
  }
  
  /* Fixing the face flags for top and bottom  */
  for(i=1; i<ncx; i++) {
    if(ybc_flag == DIRICHLET) {
      conductor[i][ncy]= 0; face[i][ncy]= DOWN; area[i][ncy]= dx*zlength;
      conductor[i][0]  = 0; face[i][0]  = UP;   area[i][0]  = dx*zlength;
    }
    else if(ybc_flag == NEUMANN) {
      if(grid_mask[i][ncy]){
	if((eps_array[i-1][ncy-1]!=0.0 && eps_array[i][ncy-1]==0.0) ||
	   (eps_array[i-1][ncy-1]==1.0 && eps_array[i][ncy-1]!=1.0)) {
	  face[i][ncy]=LEFT; area[i][ncy]= dy*zlength;
	}
	else if((eps_array[i-1][ncy-1]==0.0 && eps_array[i][ncy-1]!=0.0) ||
		(eps_array[i-1][ncy-1]!=1.0 && eps_array[i][ncy-1]==1.0)) {
	  face[i][ncy]=RIGHT; area[i][ncy]= dy*zlength;
	}
      }
      if(grid_mask[i][0]){
	if((eps_array[i-1][0]!=0.0 && eps_array[i][0]==0.0) ||
	   (eps_array[i-1][0]==1.0 && eps_array[i][0]!=1.0)) {
	  face[i][0]=LEFT; area[i][0]= dy*zlength;
	}
	else if((eps_array[i-1][0]==0.0 && eps_array[i][0]!=0.0) ||
		(eps_array[i-1][0]!=1.0 && eps_array[i][0]==1.0)) {
	  face[i][0]=RIGHT; area[i][ncy]= dy*zlength;
	}
      }
    }
  }
  
  /***************************************************/
  /* If particles are inside structures remove them  */
  
  if (nstrcmax>0) {
    for (isp=0; isp<nsp; ++isp) {
      for(n=0; n < np[isp]; n++) {
	ix= x[isp][n];
	iy= y[isp][n];
	if(cell_mask[ix][iy]) {
	  x[isp][n]  =  x[isp][np[isp]-1];
	  y[isp][n]  =  y[isp][np[isp]-1];
	  vx[isp][n] = vx[isp][np[isp]-1];
	  vy[isp][n] = vy[isp][np[isp]-1];
	  vz[isp][n] = vz[isp][np[isp]-1];
	  n--;
	  np[isp]--;
	}
      }
    }
  }
  
  /*************************************************/
  /* Setting up the energy and angular dist. stuff */
  
  if(nsp && wall_flag) {
    e_array = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    th_array= (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    fe      = (SCALAR ****)malloc(SIDES*sizeof(SCALAR ***));
    fth     = (SCALAR ****)malloc(SIDES*sizeof(SCALAR ***));
    fetot_center = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    fetot_edge   = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    fthtot_center= (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    fthtot_edge  = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    wall_q  = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    th_ave1 = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    th_ave2 = (SCALAR ***) malloc(SIDES*sizeof(SCALAR **));
    e_ave_show = (SCALAR ***)malloc(SIDES*sizeof(SCALAR **));
    th_ave_show= (SCALAR ***)malloc(SIDES*sizeof(SCALAR **));
    
    for(k=0; k<SIDES; k++) {
      e_array[k] = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      th_array[k]= (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      fe[k]      = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
      fth[k]     = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
      fetot_center[k] = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      fetot_edge[k]   = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      fthtot_center[k]= (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      fthtot_edge[k]  = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      wall_q[k]  = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      th_ave1[k] = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      th_ave2[k] = (SCALAR **) malloc(nsp*sizeof(SCALAR *));
      e_ave_show[k] = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      th_ave_show[k]= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      
      /* For each species allocate the diagnostic arrays */
      for(isp=0; isp<nsp; isp++) {
	
	/* Energy and theta arrays */
	nbin[k][isp]++;
	e_array[k][isp] = (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	th_array[k][isp]= (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	fetot_center[k][isp] = (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	fetot_edge[k][isp]   = (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	fthtot_center[k][isp]= (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	fthtot_edge[k][isp]  = (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	for(i=0; i <nbin[k][isp]; i++) {
	  e_array[k][isp][i] = i*de[k][isp];
	  th_array[k][isp][i]= i*(0.5*M_PI/(nbin[k][isp]-1));
	}
	
	/* Average angles */
	wall_q[k][isp] = (SCALAR *)malloc(ngy*sizeof(SCALAR));
	th_ave1[k][isp] = (SCALAR *)malloc(ngy*sizeof(SCALAR));
	th_ave2[k][isp] = (SCALAR *)malloc(ngy*sizeof(SCALAR));
	e_ave_show[k][isp] = (SCALAR *)malloc(ngy*sizeof(SCALAR));
	th_ave_show[k][isp]= (SCALAR *)malloc(ngy*sizeof(SCALAR));
	
	/* Energy and theta diagnostic arrays */
	fe[k][isp] = (SCALAR **)malloc(ngy*sizeof(SCALAR *));
	fth[k][isp]= (SCALAR **)malloc(ngy*sizeof(SCALAR *));
	
	for(j=0; j<ngy; j++) {
	  wall_q[k][isp][j]  = 0.0;
	  th_ave1[k][isp][j] = 0.0;
	  th_ave2[k][isp][j] = 0.0;
	  
	  fe[k][isp][j] = (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	  fth[k][isp][j]= (SCALAR *)malloc(nbin[k][isp]*sizeof(SCALAR));
	  for(i=0; i <nbin[k][isp]; i++) 
	    fe[k][isp][j][i] = fth[k][isp][j][i] = 0.0;
	}
      }
    }
  }

  if (WasDumpFileGiven) Restore(theDumpFile);
}

/***************************************************************/

void species(int isp)
{
  char aachar[80];
  int loader, fill_region;
  SCALAR xleft, xright, ylow, yhigh, initn, v0xi, v0yi, v0zi, vti, nvl, nvr;
  
  weight[isp]=1.0;
  /* Read in SPECIES parameters... */
  fscanf(InputDeck, "%s", name[isp]);
  
  while (fscanf(InputDeck, "%lf %lf %d %d %lf",
		&q[isp], &m[isp], &sp_k[isp], &maxnp[isp], &weight[isp]) <4)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %d %lf %lf %lf %lf", &loader,
		&fill_region, &xleft, &xright, &ylow, &yhigh) <6)
    fscanf(InputDeck, "%s", aachar);

  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf", &initn, &v0xi, &v0yi, &v0zi, &vti) <5)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf", &nvl, &v0x[isp][LEFT],
		&v0y[isp][LEFT], &v0z[isp][LEFT], &vt[isp][LEFT]) <5)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf", &nvr, &v0x[isp][RIGHT],
		&v0y[isp][RIGHT], &v0z[isp][RIGHT], &vt[isp][RIGHT]) <5)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %lf %lf %d %lf %lf",
		&nbin[LEFT][isp], &emin[LEFT][isp], &de[LEFT][isp],
		&nbin[RIGHT][isp],&emin[RIGHT][isp],&de[RIGHT][isp]) <6)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %lf %lf %lf %lf", &sec_flag[isp],
		&wall_sec[isp], &lhs_sec[isp], &rhs_sec[isp], &grid_sec[isp]) <5)
    fscanf(InputDeck, "%s", aachar);

  /*****************************************************/
  /* The drift and temperatures in the input file (given
     in eVs) must be converted to velocities (m/s) */
  
  v0xi = sqrt(2.0*1.602e-19*v0xi/m[isp]);
  v0yi = sqrt(2.0*1.602e-19*v0yi/m[isp]);
  v0zi = sqrt(2.0*1.602e-19*v0zi/m[isp]);
  vti  = sqrt(1.602e-19*vti/m[isp]);
  
  v0x[isp][LEFT] = sqrt(2.0*1.602e-19*v0x[isp][LEFT]/m[isp]);
  v0y[isp][LEFT] = sqrt(2.0*1.602e-19*v0y[isp][LEFT]/m[isp]);
  v0z[isp][LEFT] = sqrt(2.0*1.602e-19*v0z[isp][LEFT]/m[isp]);
  vt[isp][LEFT]  = sqrt(1.602e-19*vt[isp][LEFT]/m[isp]);
  
  v0x[isp][RIGHT] = sqrt(2.0*1.602e-19*v0x[isp][RIGHT]/m[isp]);
  v0y[isp][RIGHT] = sqrt(2.0*1.602e-19*v0y[isp][RIGHT]/m[isp]);
  v0z[isp][RIGHT] = sqrt(2.0*1.602e-19*v0z[isp][RIGHT]/m[isp]);
  vt[isp][RIGHT]  = sqrt(1.602e-19*vt[isp][RIGHT]/m[isp]);

  /* Normalization and other species related variables */
  if(sec_flag[isp]) sec_sp[isp] = sec_flag[isp]-1;
  
  qm[isp] = q[isp]/m[isp];
  q[isp] *= nc2p;
  q_per_cell[isp] = q[isp]/(zlength*dx*dy);
  dtdx[isp] = sp_k[isp]*dt/dx;
  dtdy[isp] = sp_k[isp]*dt/dy;
  
  if(bmag > 0.0) {
    vxscale[isp]= 1.0;
    vyscale[isp]= 1.0;
    vzscale[isp]= 1.0;
    ax_scale[isp] = 0.5*qm[isp]*dt;
    ay_scale[isp] = 0.5*qm[isp]*dt;
  }
  else {
    vxscale[isp]= dx/(sp_k[isp]*dt);
    vyscale[isp]= dy/(sp_k[isp]*dt);
    vzscale[isp]= zlength/(sp_k[isp]*dt);
    ax_scale[isp] = qm[isp]*sp_k[isp]*dt*dt/dx;
    ay_scale[isp] = qm[isp]*sp_k[isp]*dt*dt/dy;
  }    
  
  /* Allocate particle arrays */
  x[isp] = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
  y[isp] = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
  vx[isp]= (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
  vy[isp]= (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
  vz[isp]= (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
  
  /* Load particles if the density is non-zero */
  xleft = max(0.0, xleft);
  ylow  = max(0.0, ylow);
  xright= min(xright, xlength);
  yhigh = min(yhigh, ylength);
  if(initn>0.0)
    load(isp, initn, loader, fill_region, xleft, xright, ylow, yhigh, v0xi, v0yi, v0zi, vti);
  
  /* Calculate the number of particles to be injected from each side */ 
  enter[isp][LEFT]  = nvl*dt*sp_k[isp]*zlength*lhs_length/nc2p/weight[isp];
  if(enter[isp][LEFT]> 0.0) {
    init_vmaxwellv(isp, LEFT);  inject[isp] = 1;
  }
  
  enter[isp][RIGHT] = nvr*dt*sp_k[isp]*zlength*rhs_length/nc2p/weight[isp];
  if(enter[isp][RIGHT]> 0.0) {
    init_vmaxwellv(isp, RIGHT); inject[isp] = 1;
  }
  printf("isp=%d, enterl=%lf, enterr=%lf, inject=%d\n",
	 isp, enter[isp][LEFT], enter[isp][RIGHT], inject[isp]);
  printf("v0xi = %lf, v0yi=%lf, v0zi=%lf vti=%lf\n",
	 v0xi, v0yi, v0zi, vti);
  printf("v0xl = %lf, v0yl=%lf, v0zl=%lf vtl=%lf\n",
	 v0x[isp][LEFT], v0y[isp][LEFT], v0z[isp][LEFT], vt[isp][LEFT]);
  printf("v0xr = %lf, v0yr=%lf, v0zr=%lf vtr=%lf\n\n",
	 v0x[isp][RIGHT], v0y[isp][RIGHT], v0z[isp][RIGHT], vt[isp][RIGHT]);
}

/***************************************************************/
